#!/bin/sh
#
# sambascan2.sh - searches a network for SMB-Shares and list them.

# Copyright 2002-2004 (c) Claudio Clemens

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2, or (at your option) any later version.
 
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
  
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA.

# TODO: Mount extra checks
# TODO: Option for creating listing
# TODO: Isn't nmap to agresive?
# 

# Old-Code
# -U NOBODY% oder -N
# para lista de masters
#  smbclient -L HOST (-W WOrkgroup) 
# para cada master um smbclient -L MASTER -W WORKGROUP -> lista de hosts
#  smbclient -L LOCALHOST -N | sed -e "0,/^\tWorkgroup/d" | sed -e "1 d" -e "s/[\t ]\+\([^\t ]\+\)[\t ]\+\([^\t ]\+\)/\1:\2/g"
#  smbclient -L 10.151.4.9 -N | sed -e "0,/^\tServer/d" | sed -e "1 d" -e "/^$/Q" -e "s/\t\+\([^\t ]\+\).*/\1/g"
# for each host you can use the WG from Master,

# The line below is an alternative to the sed, but if we perl, then if should be all in perl :)
#perl -pe "s/^(.*?[^ ]) + [AHRSD]* *  ([0-9]+)  [A-Za-z]+ [A-Za-z]+ +[0-9]+ [0-9:]+ [0-9]+$/\1:\2/" samba/127.0.0.1-HALFLING-Film.txt | less

#set -x 

DEBUG=YES 
WORKDIR=$HOME/samba
TTEMP=temp.temp.$$
LTEMP1=list1.temp.$$
LTEMP2=list2.temp.$$
VERSION=0.3.4
SED=/bin/sed

function initialize {
    [ -d "$WORKDIR" ] || mkdir $WORKDIR || echo "WARNING: Workdir couldn't be created!"
    find $WORKDIR -mtime +15 -name "[0-9]*.txt.gz" | (while read a; do echo "$a" expired && rm -f "$a"; done)
}

function mytimeout () {
    smbclient "//$1/$2" -c "recurse;ls" -N -U User -d0 -D . > "$WORKDIR/$TTEMP" 2> /dev/null &
    MYPID=$!
    (sleep 120 && kill $MYPID 2> /dev/null > /dev/null) &
    wait $MYPID 2> /dev/null > /dev/null
}

function list_shares {
    SHARES=`smbclient -L $1 -N -U User | egrep "^[[:blank:]]+[^[:blank:]]+[[:blank:]]+Disk[[:blank:]]+" | $SED "s/^[[:blank:]]\+\([^[:blank:]]\+\).*/\1/g" | egrep -v "(ADMIN|IPC)\\\\$"`
    SHARES=`echo $SHARES`
    if [ "$SHARES" ] 
    then
	echo " ... Working on $2: ($SHARES)."
	for i in $SHARES
	do
	    # Here we change the SPACES back.
	    i=`echo "$i" | $SED -e "s/%20/ /g"`
	    if [ -n "$i" ]
	    then
		[ -n "$DEBUG" ] && echo -n "     Listing +($i)"
		# Because there can be some Password-shares we send stderr to null
		mytimeout "$1" "$i" 
		if (egrep "(ERRbadpw|ACCESS_DENIED|WRONG_PASSWORD|BAD_NETWORK_NAME)" "$WORKDIR/$TTEMP" > /dev/null)
		then
		    echo "... no access."
		    rm "$WORKDIR/$TTEMP"
		    # [ -n "$DEBUG" ] && echo $1:$i >> hosts.noaccess
		    continue
		fi
		if [ `wc -l < "$WORKDIR/$TTEMP"` -le 4 ]
		then 
		    echo "... empty."
		    rm "$WORKDIR/$TTEMP"
		    # [ -n "$DEBUG" ] && echo $1:$i >> hosts.emptys
		    continue
		fi
		$SED -e "s/\\\/\//g" -e "/added interface ip/d" -e "/[0-9]\+ blocks of size/d" -e "/ D[RHSA]* \+0 /d" \
		-e "/session request to .* failed (Called name not present)/d" \
		-e "s/^\(  .*\) \{2,\}A\?R\?H\?S\?\([0-9 ]\{9,\}\)  [A-Za-z 0-9:]\{24\}$/\1:\2/g" \
		-e "s/ *: */:/g" \
		"$WORKDIR/$TTEMP" > "$WORKDIR/$LTEMP1"
		rm "$WORKDIR/$TTEMP"

		# Now format the list the way we want
		# Because we need to read each line, it is a big overhead, and scanning takes now longer
		PATHNAME=/
		NOTHING=1
		while read a
		do
		    [ "$a" == "" ] && continue
		    if (echo "$a" | grep "^/" > /dev/null)
		    then
			PATHNAME="$a"/
		    else
			NOTHING=0
			echo "$1:$i:${PATHNAME}:${a}"
		    fi
		done < $WORKDIR/$LTEMP1 >> "$WORKDIR/$LTEMP2"
		[ $NOTHING -eq 1 ] && echo "$1:$i:${PATHNAME}:" >> "$WORKDIR/$LTEMP2"
		rm "$WORKDIR/$LTEMP1"
		[ -n "$DEBUG" ] && echo "... done."
	    fi
	done
	[ -s "$WORKDIR/$LTEMP2" ] && mv "$WORKDIR/$LTEMP2" "$WORKDIR/$1-$2.txt"
	rm -f "$WORKDIR/$LTEMP2"
	[ -e "$WORKDIR/$1-$2.txt" ] && gzip -f "$WORKDIR/$1-$2.txt"
    else
	echo "  ... No Shares."
    fi
    #reset -Q
}

function getlistofhosts {
    [ -n "$DEBUG" ] && echo "Getting list of hosts from net ==> $* <=="
    HOSTS=`nmap -n -sP -PT139 $* 2> /dev/null | grep ^Host | $SED "s/Host [( ]*\([0-9\.]*\).*/\1/g"`
}

function process_host {
	NAME=`nmblookup -A $1 | grep "-" | head -1 | cut -f 1 -d"<"`
	NAME=`echo $NAME`
	[ -n "$DEBUG" ] && echo -n "  Host: $NAME ($1)"
# Only Modify this section, if you know what you are doing.
# Insert your ATCION code here (e.g. scan for viruses)
	list_shares "$1" "$NAME" 
}

#### The Prog #####
# This seems not to work really OK to me. So I just put the Master I know in
# the next line

echo "sambascan2 Version: $VERSION (c) Claudio Clemens"
echo
echo "sambascan2 comes with ABSOLUTELY NO WARRANTY; for details"
echo "see the file COPYING.  This is free software, and you are welcome"
echo "to redistribute it under certain conditions."
echo

initialize

getlistofhosts $*

for i in $HOSTS
do
    process_host $i
done
# Kill leaved sleeps
kill `ps -ef | grep "sleep 120" | grep -v grep | awk '{print $2" "$3}'` 2> /dev/null > /dev/null
